@@ -1,21 +1,22 @@
 mod network;
 mod utils;
 
-use std::net::{IpAddr};
+use std::net::{IpAddr, Ipv4Addr};
 use std::process;
 use std::thread;
 
 use ipnetwork::NetworkSize;
 use pnet::datalink;
+use pnet::datalink::{MacAddr};
 use clap::{Arg, App};
 
 const FIVE_HOURS: u64 = 5 * 60 * 60; 
-const TIMEOUT_DEFAULT: u64 = 5;
+const TIMEOUT_DEFAULT: u64 = 2;
 
 fn main() {
 
     let matches = App::new("arp-scan")
-        .version("0.3.0")
+        .version("0.4.0")
         .about("A minimalistic ARP scan tool written in Rust")
         .arg(
             Arg::with_name("interface").short("i").long("interface").takes_value(true).value_name("INTERFACE_NAME").help("Network interface")
@@ -23,6 +24,12 @@ fn main() {
         .arg(
             Arg::with_name("timeout").short("t").long("timeout").takes_value(true).value_name("TIMEOUT_SECONDS").help("ARP response timeout")
         )
+        .arg(
+            Arg::with_name("source_ip").short("S").long("source-ip").takes_value(true).value_name("SOURCE_IPV4").help("Source IPv4 address for requests")
+        )
+        .arg(
+            Arg::with_name("destination_mac").short("M").long("dest-mac").takes_value(true).value_name("DESTINATION_MAC").help("Destination MAC address for requests")
+        )
         .arg(
             Arg::with_name("numeric").short("n").long("numeric").takes_value(false).help("Numeric mode, no hostname resolution")
         )
@@ -71,6 +78,32 @@ fn main() {
 
     // Hostnames will not be resolved in numeric mode
     let resolve_hostname = !matches.is_present("numeric");
+
+    let source_ipv4: Option<Ipv4Addr> = match matches.value_of("source_ip").map(|source| source.parse::<Ipv4Addr>()) {
+        Some(parsed_source) => {
+            
+            if let Err(_) = parsed_source {
+                eprintln!("Expected valid IPv4 as source IP");
+                process::exit(1);
+            }
+
+            Some(parsed_source.unwrap())
+        }, 
+        None => None
+    };
+
+    let destination_mac: Option<MacAddr> = match matches.value_of("destination_mac").map(|dest| dest.parse::<MacAddr>()) {
+        Some(mac_address) => {
+            
+            if let Err(_) = mac_address {
+                eprintln!("Expected valid MAC address as destination");
+                process::exit(1);
+            }
+
+            Some(mac_address.unwrap())
+        },
+        None => None
+    };
     
     if !utils::is_root_user() {
         eprintln!("Should run this binary as root");
@@ -99,6 +132,12 @@ fn main() {
     
     println!("");
     println!("Selected interface {} with IP {}", selected_interface.name, ip_network);
+    if let Some(forced_source_ipv4) = source_ipv4 {
+        println!("The ARP source IPv4 will be forced to {}", forced_source_ipv4);
+    }
+    if let Some(forced_destination_mac) = destination_mac {
+        println!("The ARP destination MAC will be forced to {}", forced_destination_mac);
+    }
 
     // Start ARP scan operation
     // ------------------------
@@ -123,7 +162,7 @@ fn main() {
     for ip_address in ip_network.iter() {
 
         if let IpAddr::V4(ipv4_address) = ip_address {
-            network::send_arp_request(&mut tx, selected_interface, ipv4_address);
+            network::send_arp_request(&mut tx, selected_interface, ipv4_address, source_ipv4, destination_mac);
         }
     }
 
