@@ -9,6 +9,11 @@ use pnet::packet::{MutablePacket, Packet};
 use pnet::packet::ethernet::{EthernetPacket, MutableEthernetPacket, EtherTypes};
 use pnet::packet::arp::{MutableArpPacket, ArpOperations, ArpHardwareTypes, ArpPacket};
 
+/**
+ * A target detail represents a single host on the local network with an IPv4
+ * address and a linked MAC address. Hostnames are optional since some hosts
+ * does not respond to the resolve call (or the numeric mode may be enabled).
+ */
 pub struct TargetDetails {
     pub ipv4: Ipv4Addr,
     pub mac: MacAddr,
@@ -20,12 +25,15 @@ pub struct TargetDetails {
  * interface and a target IPv4 address. The ARP request will be broadcasted to
  * the whole local network with the first valid IPv4 address on the interface.
  */
-pub fn send_arp_request(tx: &mut Box<dyn DataLinkSender>, interface: &NetworkInterface, target_ip: Ipv4Addr) {
+pub fn send_arp_request(tx: &mut Box<dyn DataLinkSender>, interface: &NetworkInterface, target_ip: Ipv4Addr, forced_source_ipv4: Option<Ipv4Addr>, forced_destination_mac: Option<MacAddr>) {
 
     let mut ethernet_buffer = [0u8; 42];
     let mut ethernet_packet = MutableEthernetPacket::new(&mut ethernet_buffer).unwrap();
 
-    let target_mac = MacAddr::broadcast();
+    let target_mac = match forced_destination_mac {
+        Some(forced_mac) => forced_mac,
+        None => MacAddr::broadcast()
+    };
     let source_mac = interface.mac.unwrap_or_else(|| {
         eprintln!("Interface should have a MAC address");
         process::exit(1);
@@ -38,6 +46,34 @@ pub fn send_arp_request(tx: &mut Box<dyn DataLinkSender>, interface: &NetworkInt
     let mut arp_buffer = [0u8; 28];
     let mut arp_packet = MutableArpPacket::new(&mut arp_buffer).unwrap();
 
+    let source_ipv4 = find_source_ip(interface, forced_source_ipv4);
+
+    arp_packet.set_hardware_type(ArpHardwareTypes::Ethernet);
+    arp_packet.set_protocol_type(EtherTypes::Ipv4);
+    arp_packet.set_hw_addr_len(6);
+    arp_packet.set_proto_addr_len(4);
+    arp_packet.set_operation(ArpOperations::Request);
+    arp_packet.set_sender_hw_addr(source_mac);
+    arp_packet.set_sender_proto_addr(source_ipv4);
+    arp_packet.set_target_hw_addr(target_mac);
+    arp_packet.set_target_proto_addr(target_ip);
+
+    ethernet_packet.set_payload(arp_packet.packet_mut());
+
+    tx.send_to(&ethernet_packet.to_immutable().packet(), Some(interface.clone()));
+}
+
+/**
+ * Find the most adequate IPv4 address on a given network interface for sending
+ * ARP requests. If the 'forced_source_ipv4' parameter is set, it will take
+ * the priority over the network interface address.
+ */
+fn find_source_ip(interface: &NetworkInterface, forced_source_ipv4: Option<Ipv4Addr>) -> Ipv4Addr {
+
+    if let Some(forced_ipv4) = forced_source_ipv4 {
+        return forced_ipv4;
+    }
+
     let source_ip = interface.ips.first().unwrap_or_else(|| {
         eprintln!("Interface should have an IP address");
         process::exit(1);
@@ -48,19 +84,7 @@ pub fn send_arp_request(tx: &mut Box<dyn DataLinkSender>, interface: &NetworkInt
         IpAddr::V6(_ipv6_addr) => None
     };
 
-    arp_packet.set_hardware_type(ArpHardwareTypes::Ethernet);
-    arp_packet.set_protocol_type(EtherTypes::Ipv4);
-    arp_packet.set_hw_addr_len(6);
-    arp_packet.set_proto_addr_len(4);
-    arp_packet.set_operation(ArpOperations::Request);
-    arp_packet.set_sender_hw_addr(source_mac);
-    arp_packet.set_sender_proto_addr(source_ipv4.unwrap());
-    arp_packet.set_target_hw_addr(target_mac);
-    arp_packet.set_target_proto_addr(target_ip);
-
-    ethernet_packet.set_payload(arp_packet.packet_mut());
-
-    tx.send_to(&ethernet_packet.to_immutable().packet(), Some(interface.clone()));
+    source_ipv4.unwrap()
 }
 
 /**
@@ -125,6 +149,10 @@ pub fn receive_arp_responses(rx: &mut Box<dyn DataLinkReceiver>, timeout_seconds
     }).collect()
 }
 
+/**
+ * Find the local hostname linked to an IPv4 address. This will perform a
+ * reverse DNS request in the local network to find the IPv4 hostname.
+ */
 fn find_hostname(ipv4: Ipv4Addr) -> Option<String> {
 
     let ip: IpAddr = ipv4.into();
@@ -142,3 +170,34 @@ fn find_hostname(ipv4: Ipv4Addr) -> Option<String> {
         Err(_) => None
     }
 }
+
+#[cfg(test)]
+mod tests {
+
+    use super::*;
+
+    #[test]
+    fn should_resolve_public_ip() {
+
+        let ipv4 = Ipv4Addr::new(1,1,1,1);
+
+        assert_eq!(find_hostname(ipv4), Some("one.one.one.one".to_string()));
+    }
+
+    #[test]
+    fn should_resolve_localhost() {
+
+        let ipv4 = Ipv4Addr::new(127,0,0,1);
+
+        assert_eq!(find_hostname(ipv4), Some("localhost".to_string()));
+    }
+
+    #[test]
+    fn should_not_resolve_unknown_ip() {
+
+        let ipv4 = Ipv4Addr::new(10,254,254,254);
+
+        assert_eq!(find_hostname(ipv4), None);
+    }
+
+}
